@@ -2,7 +2,7 @@
 # Pitivi video editor
 # Copyright (c) 2005-2009 Edward Hervey <bilboed@bilboed.com>
 # Copyright (c) 2008-2009 Alessandro Decina <alessandro.d@gmail.com>
-# Copyright (c) 2014 Alexandru Băluț<alexandru.balut@gmail.com>
+# Copyright (c) 2014 Alexandru Băluț <alexandru.balut@gmail.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -15,25 +15,23 @@
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, write to the
-# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
-# Boston, MA 02110-1301, USA.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
 import os
 import time
 from gettext import gettext as _
 
 from gi.repository import Gio
+from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gst
 from gi.repository import Gtk
 
 from pitivi.configure import RELEASES_URL
 from pitivi.configure import VERSION
-from pitivi.dialogs.startupwizard import StartUpWizard
 from pitivi.effects import EffectsManager
 from pitivi.mainwindow import MainWindow
+from pitivi.pluginmanager import PluginManager
 from pitivi.project import ProjectManager
-from pitivi.settings import get_dir
 from pitivi.settings import GlobalSettings
 from pitivi.settings import xdg_cache_home
 from pitivi.shortcuts import ShortcutsManager
@@ -57,19 +55,20 @@ class Pitivi(Gtk.Application, Loggable):
         action_log (UndoableActionLog): The undo/redo log for the current project.
         effects (EffectsManager): The effects which can be applied to a clip.
         gui (MainWindow): The main window of the app.
+        recent_manager (Gtk.RecentManager): Manages recently used projects.
         project_manager (ProjectManager): The holder of the current project.
         settings (GlobalSettings): The application-wide settings.
         system (pitivi.utils.system.System): The system running the app.
     """
 
     __gsignals__ = {
-        "version-info-received": (GObject.SIGNAL_RUN_LAST, None, (object,))
+        "version-info-received": (GObject.SignalFlags.RUN_LAST, None, (object,))
     }
 
     def __init__(self):
         Gtk.Application.__init__(self,
-                                 application_id="org.pitivi",
-                                 flags=Gio.ApplicationFlags.HANDLES_OPEN)
+                                 application_id="org.pitivi.Pitivi",
+                                 flags=Gio.ApplicationFlags.NON_UNIQUE | Gio.ApplicationFlags.HANDLES_OPEN)
         Loggable.__init__(self)
 
         self.settings = None
@@ -83,7 +82,7 @@ class Pitivi(Gtk.Application, Loggable):
         self._last_action_time = Gst.util_get_timestamp()
 
         self.gui = None
-        self.__welcome_wizard = None
+        self.recent_manager = Gtk.RecentManager.get_default()
         self.__inhibit_cookies = {}
 
         self._version_information = {}
@@ -108,7 +107,8 @@ class Pitivi(Gtk.Application, Loggable):
             # We need to make sure that the waiting time was more than 50 ms.
             st = Gst.Structure.new_empty("wait")
             st["duration"] = float((now - self._last_action_time) / Gst.SECOND)
-            self._scenario_file.write(st.to_string() + "\n")
+            self._scenario_file.write(st.to_string())
+            self._scenario_file.write("\n")
             self._last_action_time = now
 
         if not isinstance(action, Gst.Structure):
@@ -120,15 +120,15 @@ class Pitivi(Gtk.Application, Loggable):
 
             action = structure
 
-        self._scenario_file.write(action.to_string() + "\n")
+        self._scenario_file.write(action.to_string())
+        self._scenario_file.write("\n")
         self._scenario_file.flush()
 
     def do_startup(self):
         Gtk.Application.do_startup(self)
 
         # Init logging as early as possible so we can log startup code
-        enable_color = not os.environ.get(
-            'PITIVI_DEBUG_NO_COLOR', '0') in ('', '1')
+        enable_color = os.environ.get('PITIVI_DEBUG_NO_COLOR', '0') not in ('', '1')
         # Let's show a human-readable Pitivi debug output by default, and only
         # show a crazy unreadable mess when surrounded by gst debug statements.
         enable_crack_output = "GST_DEBUG" in os.environ
@@ -136,48 +136,51 @@ class Pitivi(Gtk.Application, Loggable):
 
         self.info('starting up')
         self._setup()
-        self._checkVersion()
+        self._check_version()
 
     def _setup(self):
+        # pylint: disable=attribute-defined-outside-init
         self.settings = GlobalSettings()
         self.threads = ThreadMaster()
         self.effects = EffectsManager()
         self.proxy_manager = ProxyManager(self)
         self.system = get_system()
+        self.plugin_manager = PluginManager(self)
 
-        self.project_manager.connect(
-            "new-project-loading", self._newProjectLoadingCb)
-        self.project_manager.connect(
-            "new-project-loaded", self._newProjectLoaded)
-        self.project_manager.connect("project-closed", self._projectClosed)
+        self.project_manager.connect("new-project-loaded", self._new_project_loaded_cb)
+        self.project_manager.connect_after("project-closed", self._project_closed_cb)
+        self.project_manager.connect("project-saved", self.__project_saved_cb)
 
-        self._createActions()
-        self._syncDoUndo()
+        self._create_actions()
+        self._sync_do_undo()
 
-    def _createActions(self):
+    def _create_actions(self):
         self.shortcuts.register_group("app", _("General"), position=10)
+
+        # pylint: disable=attribute-defined-outside-init
         self.undo_action = Gio.SimpleAction.new("undo", None)
-        self.undo_action.connect("activate", self._undoCb)
+        self.undo_action.connect("activate", self._undo_cb)
         self.add_action(self.undo_action)
-        self.shortcuts.add("app.undo", ["<Primary>z"],
+        self.shortcuts.add("app.undo", ["<Primary>z"], self.undo_action,
                            _("Undo the most recent action"))
 
         self.redo_action = Gio.SimpleAction.new("redo", None)
-        self.redo_action.connect("activate", self._redoCb)
+        self.redo_action.connect("activate", self._redo_cb)
         self.add_action(self.redo_action)
-        self.shortcuts.add("app.redo", ["<Primary><Shift>z"],
+        self.shortcuts.add("app.redo", ["<Primary><Shift>z"], self.redo_action,
                            _("Redo the most recent action"))
 
         self.quit_action = Gio.SimpleAction.new("quit", None)
-        self.quit_action.connect("activate", self._quitCb)
+        self.quit_action.connect("activate", self._quit_cb)
         self.add_action(self.quit_action)
-        self.shortcuts.add("app.quit", ["<Primary>q"], _("Quit"))
+        self.shortcuts.add("app.quit", ["<Primary>q"], self.quit_action, _("Quit"))
 
         self.show_shortcuts_action = Gio.SimpleAction.new("shortcuts_window", None)
         self.show_shortcuts_action.connect("activate", self._show_shortcuts_cb)
         self.add_action(self.show_shortcuts_action)
         self.shortcuts.add("app.shortcuts_window",
                            ["<Primary>F1", "<Primary>question"],
+                           self.show_shortcuts_action,
                            _("Show the Shortcuts Window"))
 
     def do_activate(self):
@@ -196,32 +199,24 @@ class Pitivi(Gtk.Application, Loggable):
                 self.gui.present()
             # No need to show the welcome wizard.
             return
-        self.createMainWindow()
-        self.welcome_wizard.show()
 
-    @property
-    def welcome_wizard(self):
-        if not self.__welcome_wizard:
-            self.__welcome_wizard = StartUpWizard(self)
-        return self.__welcome_wizard
+        self.create_main_window()
+        self.gui.show_perspective(self.gui.greeter)
 
-    def createMainWindow(self):
+    def create_main_window(self):
         if self.gui:
             return
         self.gui = MainWindow(self)
+        self.gui.setup_ui()
         self.add_window(self.gui)
-        self.gui.checkScreenConstraints()
-        # We might as well show it.
-        self.gui.show()
 
     def do_open(self, giofiles, unused_count, unused_hint):
         assert giofiles
-        self.createMainWindow()
+        self.create_main_window()
         if len(giofiles) > 1:
-            self.warning(
-                "Can open only one project file at a time. Ignoring the rest!")
+            self.warning("Opening only one project at a time. Ignoring the rest!")
         project_file = giofiles[0]
-        self.project_manager.loadProject(quote_uri(project_file.get_uri()))
+        self.project_manager.load_project(quote_uri(project_file.get_uri()))
         return True
 
     def shutdown(self):
@@ -232,20 +227,18 @@ class Pitivi(Gtk.Application, Loggable):
         """
         self.debug("shutting down")
         # Refuse to close if we are not done with the current project.
-        if not self.project_manager.closeRunningProject():
+        if not self.project_manager.close_running_project():
             self.warning(
                 "Not closing since running project doesn't want to close")
             return False
-        if self.welcome_wizard:
-            self.welcome_wizard.hide()
         if self.gui:
             self.gui.destroy()
-        self.threads.stopAllThreads()
-        self.settings.storeSettings()
+        self.threads.wait_all_threads()
+        self.settings.store_settings()
         self.quit()
         return True
 
-    def _setScenarioFile(self, uri):
+    def _set_scenario_file(self, uri):
         if uri:
             project_path = path_from_uri(uri)
         else:
@@ -254,7 +247,7 @@ class Pitivi(Gtk.Application, Loggable):
         if 'PITIVI_SCENARIO_FILE' in os.environ:
             scenario_path = os.environ['PITIVI_SCENARIO_FILE']
         else:
-            cache_dir = get_dir(os.path.join(xdg_cache_home(), "scenarios"))
+            cache_dir = xdg_cache_home("scenarios")
             scenario_name = str(time.strftime("%Y%m%d-%H%M%S"))
             if project_path:
                 scenario_name += os.path.splitext(project_path.replace(os.sep, "_"))[0]
@@ -270,34 +263,49 @@ class Pitivi(Gtk.Application, Loggable):
                 self.write_action("load-project",
                                   serialized_content=content)
 
-    def _newProjectLoadingCb(self, unused_project_manager, project):
-        self._setScenarioFile(project.get_uri())
+    def _new_project_loaded_cb(self, unused_project_manager, project):
+        uri = project.get_uri()
+        if uri:
+            # We remove the project from recent projects list
+            # and then re-add it to this list to make sure it
+            # gets positioned at the top of the recent projects list.
+            try:
+                self.recent_manager.remove_item(uri)
+            except GLib.Error as e:
+                if e.domain != "gtk-recent-manager-error-quark":
+                    raise e
+            self.recent_manager.add_item(uri)
 
-    def _newProjectLoaded(self, unused_project_manager, project):
         self.action_log = UndoableActionLog()
         self.action_log.connect("pre-push", self._action_log_pre_push_cb)
-        self.action_log.connect("commit", self._actionLogCommit)
+        self.action_log.connect("commit", self._action_log_commit)
         self.action_log.connect("move", self._action_log_move_cb)
-
         self.project_observer = ProjectObserver(project, self.action_log)
 
-    def _projectClosed(self, unused_project_manager, project):
+        self._set_scenario_file(project.get_uri())
+
+    def __project_saved_cb(self, unused_project_manager, unused_project, uri):
+        if uri:
+            self.recent_manager.add_item(uri)
+
+    def _project_closed_cb(self, unused_project_manager, project):
         if project.loaded:
             self.action_log = None
-            self._syncDoUndo()
+            self._sync_do_undo()
 
         if self._scenario_file:
             self.write_action("stop")
             self._scenario_file.close()
             self._scenario_file = None
 
-    def _checkVersion(self):
+    def _check_version(self):
         """Checks online for new versions of the app."""
         self.info("Requesting version information async")
         giofile = Gio.File.new_for_uri(RELEASES_URL)
         giofile.load_contents_async(None, self._version_info_received_cb, None)
 
     def _version_info_received_cb(self, giofile, result, user_data):
+        # pylint: disable=broad-except
         try:
             raw = giofile.load_contents_finish(result)[1]
             if not isinstance(raw, str):
@@ -318,14 +326,14 @@ class Pitivi(Gtk.Application, Loggable):
                     current_version = version
                     self.info("Latest software version is %s", current_version)
 
-            VERSION_split = [int(i) for i in VERSION.split(".")]
+            version_split = [int(i) for i in VERSION.split(".")]
             current_version_split = [int(i)
                                      for i in current_version.split(".")]
-            if VERSION_split > current_version_split:
+            if version_split > current_version_split:
                 status = "CURRENT"
                 self.info(
                     "Running version %s, which is newer than the latest known version. Considering it as the latest current version.", VERSION)
-            elif status is "UNSUPPORTED":
+            elif status == "UNSUPPORTED":
                 self.warning(
                     "Using an outdated version of Pitivi (%s)", VERSION)
 
@@ -335,45 +343,41 @@ class Pitivi(Gtk.Application, Loggable):
         except Exception as e:
             self.warning("Version info could not be read: %s", e)
 
-    def isLatest(self):
+    def is_latest(self):
         """Whether the app's version is the latest as far as we know."""
         status = self._version_information.get("status")
         return status is None or status.upper() == "CURRENT"
 
-    def getLatest(self):
+    def get_latest(self):
         """Get the latest version of the app or None."""
         return self._version_information.get("current")
 
-    def _quitCb(self, unused_action, unused_param):
+    def _quit_cb(self, unused_action, unused_param):
         self.shutdown()
 
-    def _undoCb(self, unused_action, unused_param):
+    def _undo_cb(self, unused_action, unused_param):
         self.action_log.undo()
 
-    def _redoCb(self, unused_action, unused_param):
+    def _redo_cb(self, unused_action, unused_param):
         self.action_log.redo()
 
     def _show_shortcuts_cb(self, unused_action, unused_param):
         show_shortcuts(self)
 
     def _action_log_pre_push_cb(self, unused_action_log, action):
-        try:
-            st = action.asScenarioAction()
-        except NotImplementedError:
-            self.warning("No serialization method for action %s", action)
-            return
-        if st:
-            self.write_action(st)
+        scenario_action = action.as_scenario_action()
+        if scenario_action:
+            self.write_action(scenario_action)
 
-    def _actionLogCommit(self, action_log, unused_stack):
+    def _action_log_commit(self, action_log, unused_stack):
         if action_log.is_in_transaction():
             return
-        self._syncDoUndo()
+        self._sync_do_undo()
 
     def _action_log_move_cb(self, action_log, unused_stack):
-        self._syncDoUndo()
+        self._sync_do_undo()
 
-    def _syncDoUndo(self):
+    def _sync_do_undo(self):
         can_undo = self.action_log and bool(self.action_log.undo_stacks)
         self.undo_action.set_enabled(bool(can_undo))
 
@@ -384,10 +388,10 @@ class Pitivi(Gtk.Application, Loggable):
             return
 
         dirty = self.action_log and self.action_log.dirty()
-        self.project_manager.current_project.setModificationState(dirty)
+        self.project_manager.current_project.set_modification_state(dirty)
         # In the tests we do not want to create any gui
         if self.gui is not None:
-            self.gui.showProjectStatus()
+            self.gui.editor.show_project_status()
 
     def simple_inhibit(self, reason, flags):
         """Informs the session manager about actions to be inhibited.
