@@ -28,13 +28,13 @@ from pitivi.clipproperties import ClipProperties
 from pitivi.configure import APPNAME
 from pitivi.configure import get_ui_dir
 from pitivi.dialogs.missingasset import MissingAssetDialog
+from pitivi.dialogs.projectsettings import ProjectSettingsDialog
 from pitivi.editorstate import EditorState
 from pitivi.effects import EffectListWidget
 from pitivi.interactiveintro import InteractiveIntro
 from pitivi.mediafilespreviewer import PreviewWidget
 from pitivi.medialibrary import MediaLibraryWidget
 from pitivi.perspective import Perspective
-from pitivi.project import ProjectSettingsDialog
 from pitivi.settings import GlobalSettings
 from pitivi.tabsmanager import BaseTabs
 from pitivi.timeline.previewers import ThumbnailCache
@@ -111,6 +111,7 @@ class EditorPerspective(Perspective, Loggable):
 
     def refresh(self):
         """Refreshes the perspective."""
+        self.timeline_ui.restore_state()
         self.focus_timeline()
 
     def __setup_css(self):
@@ -127,6 +128,9 @@ class EditorPerspective(Perspective, Loggable):
             # Nothing to work with, Pitivi is starting up.
             return
 
+        if self.app.project_manager.current_project.pipeline.rendering():
+            return
+
         # Commit the timeline so its nested timelines assets are refreshed.
         ges_timeline.commit()
 
