@@ -32,15 +32,18 @@ from gi.repository import Gtk
 
 from pitivi import configure
 from pitivi.check import MISSING_SOFT_DEPS
+from pitivi.dialogs.projectsettings import ProjectSettingsDialog
 from pitivi.utils.loggable import Loggable
 from pitivi.utils.misc import cmp
+from pitivi.utils.misc import is_pathname_valid
 from pitivi.utils.misc import path_from_uri
 from pitivi.utils.misc import show_user_manual
 from pitivi.utils.ripple_update_group import RippleUpdateGroup
 from pitivi.utils.ui import AUDIO_CHANNELS
-from pitivi.utils.ui import AUDIO_RATES
 from pitivi.utils.ui import beautify_eta
+from pitivi.utils.ui import create_audio_rates_model
 from pitivi.utils.ui import create_frame_rates_model
+from pitivi.utils.ui import filter_unsupported_media_files
 from pitivi.utils.ui import get_combo_value
 from pitivi.utils.ui import set_combo_value
 from pitivi.utils.widgets import GstElementSettingsDialog
@@ -814,17 +817,11 @@ class RenderDialog(Loggable):
         self._create_ui()
 
         # Directory and Filename
-        self.filebutton.set_current_folder(self.app.settings.lastExportFolder)
         if not self.project.name:
             self._update_filename(_("Untitled"))
         else:
             self._update_filename(self.project.name)
 
-        # Add a shortcut for the project folder (if saved)
-        if self.project.uri:
-            shortcut = os.path.dirname(self.project.uri)
-            self.filebutton.add_shortcut_folder_uri(shortcut)
-
         self._setting_encoding_profile = False
 
         # We store these so that when the user tries various container formats,
@@ -834,19 +831,14 @@ class RenderDialog(Loggable):
         self.preferred_aencoder = self.project.aencoder
         self.__replaced_assets = {}
 
-        self.channels_combo.set_model(AUDIO_CHANNELS)
-        self.sample_rate_combo.set_model(AUDIO_RATES)
-        self.__initialize_muxers_model()
-        self._display_settings()
         self._display_render_settings()
 
         self.window.connect("delete-event", self._delete_event_cb)
-        self.project.connect("rendering-settings-changed",
-                             self._rendering_settings_changed_cb)
+        self.project.connect("video-size-changed", self._project_video_size_changed_cb)
 
         self.presets_manager.connect("profile-updated", self._presets_manager_profile_updated_cb)
 
-        preset_item = self.presets_manager.initial_preset()
+        preset_item: PresetItem = self.presets_manager.initial_preset()
         if preset_item:
             if self.apply_preset(preset_item):
                 self.apply_vcodecsettings_quality(Quality.MEDIUM)
@@ -1022,7 +1014,7 @@ class RenderDialog(Loggable):
 
             self.preset_popover.hide()
 
-    def apply_preset(self, preset_item):
+    def apply_preset(self, preset_item: PresetItem):
         old_profile = self.project.container_profile
         profile = preset_item.profile.copy()
         if not self._set_encoding_profile(profile):
@@ -1038,15 +1030,10 @@ class RenderDialog(Loggable):
     def _preset_selection_menubutton_clicked_cb(self, button):
         self.preset_popover.show_all()
 
-    def _rendering_settings_changed_cb(self, unused_project, unused_item):
+    def _project_video_size_changed_cb(self, project):
         """Handles Project metadata changes."""
         self.update_resolution()
 
-    def __initialize_muxers_model(self):
-        # By default show only supported muxers and encoders.
-        model = self.create_combobox_model(Encoders().muxers)
-        self.muxer_combo.set_model(model)
-
     def create_combobox_model(self, factories):
         """Creates a model for a combobox showing factories.
 
@@ -1122,6 +1109,8 @@ class RenderDialog(Loggable):
         self.scale_spinbutton.set_value(self.project.render_scale)
 
         # Muxer settings
+        model = self.create_combobox_model(Encoders().muxers)
+        self.muxer_combo.set_model(model)
         # This will trigger an update of the codec comboboxes.
         muxer = Encoders().factories_by_name.get(self.project.muxer)
         if muxer:
@@ -1133,35 +1122,30 @@ class RenderDialog(Loggable):
 
     def _check_filename(self):
         """Displays a warning if the file path already exists."""
-        path = self.filebutton.get_current_folder()
-        if not path:
-            # This happens when the window is initialized.
-            return
-
-        filename = self.fileentry.get_text()
-
-        # Characters that cause pipeline failure.
-        blacklist = ["/"]
-        invalid_chars = "".join([ch for ch in blacklist if ch in filename])
-
-        warning_icon = "dialog-warning"
-        self._is_filename_valid = True
-        if not filename:
+        filepath = self.fileentry.get_text()
+        if not filepath:
             tooltip_text = _("A file name is required.")
             self._is_filename_valid = False
-        elif os.path.exists(os.path.join(path, filename)):
-            tooltip_text = _("This file already exists.\n"
-                             "If you don't want to overwrite it, choose a "
-                             "different file name or folder.")
-        elif invalid_chars:
-            tooltip_text = _("Remove invalid characters from the filename: %s") % invalid_chars
-            self._is_filename_valid = False
         else:
-            warning_icon = None
-            tooltip_text = None
+            filepath = os.path.realpath(filepath)
+            if os.path.isdir(filepath):
+                tooltip_text = _("A file name is required.")
+                self._is_filename_valid = False
+            elif os.path.exists(filepath):
+                tooltip_text = _("This file already exists.\n"
+                                 "If you don't want to overwrite it, choose a "
+                                 "different file name or folder.")
+                self._is_filename_valid = True
+            elif not is_pathname_valid(filepath):
+                tooltip_text = _("Invalid file path")
+                self._is_filename_valid = False
+            else:
+                tooltip_text = None
+                self._is_filename_valid = True
 
-        self.fileentry.set_icon_from_icon_name(1, warning_icon)
-        self.fileentry.set_icon_tooltip_text(1, tooltip_text)
+        warning_icon = "dialog-warning" if tooltip_text else None
+        self.fileentry.set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, warning_icon)
+        self.fileentry.set_icon_tooltip_text(Gtk.EntryIconPosition.SECONDARY, tooltip_text)
         self.__update_render_button_sensitivity()
 
     def _get_filesize_estimate(self):
@@ -1199,7 +1183,8 @@ class RenderDialog(Loggable):
             name = "%s%s%s" % (basename, os.path.extsep, extension)
         else:
             name = basename
-        self.fileentry.set_text(name)
+
+        self.fileentry.set_text(os.path.join(self.app.settings.lastExportFolder, name))
 
     def _update_valid_restriction_values(self, caps, combo, caps_template,
                                          model, combo_value,
@@ -1214,8 +1199,13 @@ class RenderDialog(Loggable):
         reduced_model = Gtk.ListStore(*model_headers)
         reduced = []
         for name, value in dict(model).items():
-            ecaps = Gst.Caps(caps_template_expander(caps_template, value))
-            if not caps.intersect(ecaps).is_empty():
+            caps_raw = caps_template_expander(caps_template, value)
+            ecaps = Gst.Caps(caps_raw)
+            if caps.intersect(ecaps).is_empty():
+                self.warning(
+                    "Ignoring value because not supported by the encoder: %s",
+                    caps_raw)
+            else:
                 reduced.append((name, value))
 
         for value in sorted(reduced, key=lambda v: float(v[1])):
@@ -1232,9 +1222,10 @@ class RenderDialog(Loggable):
                     if t.direction == Gst.PadDirection.SINK][0]
 
         caps = template.static_caps.get()
+        model = create_audio_rates_model(self.project.audiorate)
         self._update_valid_restriction_values(caps, self.sample_rate_combo,
                                               "audio/x-raw,rate=(int)%d",
-                                              AUDIO_RATES,
+                                              model,
                                               self.project.audiorate)
 
         self._update_valid_restriction_values(caps, self.channels_combo,
@@ -1249,10 +1240,10 @@ class RenderDialog(Loggable):
         template = [t for t in factory.get_static_pad_templates()
                     if t.direction == Gst.PadDirection.SINK][0]
 
+        caps = template.static_caps.get()
+
         fr_datum = (self.project.videorate.num, self.project.videorate.denom)
         model = create_frame_rates_model(fr_datum)
-
-        caps = template.static_caps.get()
         self._update_valid_restriction_values(
             caps, self.frame_rate_combo,
             "video/x-raw,framerate=(GstFraction)%d/%d", model,
@@ -1437,11 +1428,10 @@ class RenderDialog(Loggable):
 
         # Use HQ Proxy (or equivalent) only for unsupported assets
         if self.__automatically_use_proxies.get_active():
-            if self.app.proxy_manager.is_asset_format_well_supported(
-                    asset_target):
+            if self.app.proxy_manager.is_asset_format_well_supported(asset_target):
                 return asset_target
-            else:
-                proxy_unsupported = True
+
+            proxy_unsupported = True
 
         # Use HQ Proxy (or equivalent) whenever available
         if self.__always_use_proxies.get_active() or proxy_unsupported:
@@ -1489,12 +1479,43 @@ class RenderDialog(Loggable):
             getattr(self.project, media_type + "_profile").set_format(caps)
         self.dialog.window.destroy()
 
+    def _select_file_clicked_cb(self, unused_button):
+        chooser = Gtk.FileChooserNative.new(
+            _("Select file path to render"),
+            self.window,
+            Gtk.FileChooserAction.SAVE,
+            None, None)
+
+        file_filter = Gtk.FileFilter()
+        file_filter.set_name(_("Supported file formats"))
+        file_filter.add_custom(Gtk.FileFilterFlags.URI | Gtk.FileFilterFlags.MIME_TYPE,
+                               filter_unsupported_media_files)
+        chooser.add_filter(file_filter)
+        chooser.set_current_folder(self.app.settings.lastExportFolder)
+        # Add a shortcut for the project folder (if saved)
+        if self.project.uri:
+            shortcut = os.path.dirname(self.project.uri)
+            chooser.add_shortcut_folder_uri(shortcut)
+
+        response = chooser.run()
+        if response == Gtk.ResponseType.DELETE_EVENT:
+            # This happens because Gtk.FileChooserNative is confused because we
+            # added a filter but since it's ignored it complains there is none.
+            # Try again without the filter.
+            chooser.remove_filter(file_filter)
+            response = chooser.run()
+
+        if response == Gtk.ResponseType.ACCEPT:
+            self.app.settings.lastExportFolder = chooser.get_current_folder()
+            self.fileentry.set_text(os.path.join(chooser.get_filename()))
+
     def _render_button_clicked_cb(self, unused_button):
         """Starts the rendering process."""
         self.__replace_proxies()
         self.__unset_effect_preview_props()
-        self.outfile = os.path.join(self.filebutton.get_uri(),
-                                    self.fileentry.get_text())
+        filename = os.path.realpath(self.fileentry.get_text())
+        self.outfile = Gst.filename_to_uri(filename)
+        self.app.settings.lastExportFolder = os.path.dirname(filename)
         self.progress = RenderingProgressDialog(self.app, self)
         # Hide the rendering settings dialog while rendering
         self.window.hide()
@@ -1513,7 +1534,6 @@ class RenderDialog(Loggable):
         self.project.pipeline.connect("position", self._update_position_cb)
         # Force writing the config now, or the path will be reset
         # if the user opens the rendering dialog again
-        self.app.settings.lastExportFolder = self.filebutton.get_current_folder()
         self.app.settings.store_settings()
 
     def _close_button_clicked_cb(self, unused_button):
@@ -1528,10 +1548,7 @@ class RenderDialog(Loggable):
     def _container_context_help_clicked_cb(self, unused_button):
         show_user_manual("codecscontainers")
 
-    def _current_folder_changed_cb(self, *unused_args):
-        self._check_filename()
-
-    def _filename_changed_cb(self, *unused_args):
+    def _fileentry_changed_cb(self, unused_entry):
         self._check_filename()
 
     # Periodic (timer) callbacks
@@ -1539,6 +1556,7 @@ class RenderDialog(Loggable):
         if self._rendering_is_paused:
             # Do nothing until we resume rendering
             return True
+
         if self._is_rendering:
             if self.current_position:
                 timediff = time.time() - self._time_started - self._time_spent_paused
@@ -1549,10 +1567,10 @@ class RenderDialog(Loggable):
                 if estimate:
                     self.progress.update_progressbar_eta(estimate)
             return True
-        else:
-            self._time_estimate_timer = None
-            self.debug("Stopping the ETA timer")
-            return False
+
+        self._time_estimate_timer = None
+        self.debug("Stopping the ETA timer")
+        return False
 
     def _update_filesize_estimate_cb(self):
         if self._rendering_is_paused:
@@ -1662,7 +1680,6 @@ class RenderDialog(Loggable):
         self.resolution_label.set_text("%d×%d" % (width, height))
 
     def _project_settings_button_clicked_cb(self, unused_button):
-        from pitivi.project import ProjectSettingsDialog
         dialog = ProjectSettingsDialog(self.window, self.project, self.app)
         dialog.window.run()
         self._display_settings()
