@@ -16,6 +16,8 @@
 # License along with this program; if not, see <http://www.gnu.org/licenses/>.
 import os
 from gettext import gettext as _
+from typing import List
+from typing import Optional
 
 from gi.repository import Gdk
 from gi.repository import GES
@@ -473,7 +475,7 @@ class Timeline(Gtk.EventBox, Zoomable, Loggable):
             self.ges_timeline.ui = None
             self.ges_timeline = None
 
-        if self._project:
+        if self._project and self._project.pipeline:
             self._project.pipeline.disconnect_by_func(self._position_cb)
 
         self._project = project
@@ -1445,7 +1447,7 @@ class TimelineContainer(Gtk.Grid, Zoomable, Loggable):
         Gtk.Grid.__init__(self)
         Loggable.__init__(self)
 
-        self.app = app
+        self.app: Gtk.Application = app
         self.editor_state = editor_state
         self._settings = self.app.settings
         self.shift_mask = False
@@ -1582,11 +1584,16 @@ class TimelineContainer(Gtk.Grid, Zoomable, Loggable):
 
             self.timeline.update_snapping_distance()
             self.markers.markers_container = project.ges_timeline.get_marker_list("markers")
+            self.editor_state.set_project(project)
+            self.restore_state()
 
     def restore_state(self):
         if self.state_restored:
             return
 
+        if not self._project or not self.get_realized():
+            return
+
         # One attempt is enough.
         self.state_restored = True
 
@@ -1612,6 +1619,10 @@ class TimelineContainer(Gtk.Grid, Zoomable, Loggable):
             # TODO: Figure out why self.scroll_to_pixel(scroll) which calls _scroll_to_pixel directly does not work.
             GLib.idle_add(self._scroll_to_pixel, scroll)
 
+    def do_realize(self):
+        Gtk.Widget.do_realize(self)
+        self.restore_state()
+
     def update_actions(self):
         selection = self.timeline.selection
         selection_non_empty = bool(selection)
@@ -1768,14 +1779,42 @@ class TimelineContainer(Gtk.Grid, Zoomable, Loggable):
         group.add_action(self.seek_forward_clip_action)
         self.app.shortcuts.add("timeline.seek-forward-clip", ["<Primary>Right"],
                                self.seek_forward_clip_action,
-                               _("Seeks to the first clip edge after the playhead."))
+                               _("Seek to the first clip edge after the playhead"))
 
         self.seek_backward_clip_action = Gio.SimpleAction.new("seek-backward-clip", None)
         self.seek_backward_clip_action.connect("activate", self._seek_backward_clip_cb)
         group.add_action(self.seek_backward_clip_action)
         self.app.shortcuts.add("timeline.seek-backward-clip", ["<Primary>Left"],
                                self.seek_backward_clip_action,
-                               _("Seeks to the first clip edge before the playhead."))
+                               _("Seek to the first clip edge before the playhead"))
+
+        self.shift_forward_action = Gio.SimpleAction.new("shift-forward", None)
+        self.shift_forward_action.connect("activate", self._shift_forward_cb)
+        group.add_action(self.shift_forward_action)
+        self.app.shortcuts.add("timeline.shift-forward", ["<Primary><Shift>Right"],
+                               self.shift_forward_action,
+                               _("Shift selected clips one frame forward"))
+
+        self.shift_backward_action = Gio.SimpleAction.new("shift-backward", None)
+        self.shift_backward_action.connect("activate", self._shift_backward_cb)
+        group.add_action(self.shift_backward_action)
+        self.app.shortcuts.add("timeline.shift-backward", ["<Primary><Shift>Left"],
+                               self.shift_backward_action,
+                               _("Shift selected clips one frame backward"))
+
+        self.snap_clips_forward_action = Gio.SimpleAction.new("snap-clips-forward", None)
+        self.snap_clips_forward_action.connect("activate", self._snap_clips_forward_cb)
+        group.add_action(self.snap_clips_forward_action)
+        self.app.shortcuts.add("timeline.snap-clips-forward", ["<Alt>s"],
+                               self.snap_clips_forward_action,
+                               _("Snap selected clips to the next clip"))
+
+        self.snap_clips_backward_action = Gio.SimpleAction.new("snap-clips-backward", None)
+        self.snap_clips_backward_action.connect("activate", self._snap_clips_backward_cb)
+        group.add_action(self.snap_clips_backward_action)
+        self.app.shortcuts.add("timeline.snap-clips-backward", ["<Alt>a"],
+                               self.snap_clips_backward_action,
+                               _("Snap selected clips to the previous clip"))
 
         self.add_effect_action = Gio.SimpleAction.new("add-effect", None)
         self.add_effect_action.connect("activate", self.__add_effect_cb)
@@ -2000,16 +2039,22 @@ class TimelineContainer(Gtk.Grid, Zoomable, Loggable):
         with self.app.action_log.started("paste",
                                          finalizing_action=CommitTimelineFinalizingAction(self._project.pipeline),
                                          toplevel=True):
-            copied_group_shallow_copy = self.__copied_group.paste(position)
-            if not copied_group_shallow_copy:
+            pasted_group = self.__copied_group.paste(position)
+            if not pasted_group:
                 self.info("The paste is not possible at position: %s", position)
                 return
 
+            # Need to save this as the .ungroup() below changes the duration.
+            duration = pasted_group.duration
+            self.timeline.selection.select(pasted_group.children)
             try:
-                self.__copied_group = copied_group_shallow_copy.copy(True)
+                # We need to recreate the copied group as pasting destroys it.
+                self.__copied_group = pasted_group.copy(True)
                 self.__copied_group.props.serialize = False
             finally:
-                copied_group_shallow_copy.ungroup(recursive=False)
+                pasted_group.ungroup(recursive=False)
+        # Seek to the end of the pasted clip(s) for convenience.
+        self._project.pipeline.simple_seek(position + duration)
 
     def __add_layer_cb(self, unused_action, unused_parameter):
         with self.app.action_log.started("add layer",
@@ -2018,7 +2063,7 @@ class TimelineContainer(Gtk.Grid, Zoomable, Loggable):
             priority = len(self.ges_timeline.get_layers())
             self.timeline.create_layer(priority)
 
-    def first_clip_edge(self, before=None, after=None):
+    def first_clip_edge(self, layers: Optional[List[GES.Layer]] = None, before: Optional[int] = None, after: Optional[int] = None) -> Optional[int]:
         assert (after is not None) != (before is not None)
 
         if after is not None:
@@ -2033,7 +2078,9 @@ class TimelineContainer(Gtk.Grid, Zoomable, Loggable):
         if start >= end:
             return None
 
-        for layer in self.ges_timeline.layers:
+        if not layers:
+            layers = self.ges_timeline.layers
+        for layer in layers:
             clips = layer.get_clips_in_interval(start, end)
             for clip in clips:
                 if clip.start > start:
@@ -2065,7 +2112,7 @@ class TimelineContainer(Gtk.Grid, Zoomable, Loggable):
         self.timeline.scroll_to_playhead(align=Gtk.Align.CENTER, when_not_in_view=True)
 
     def __add_effect_cb(self, unused_action, unused_parameter):
-        clip = self.timeline.selection.getSingleClip()
+        clip = self.timeline.selection.get_single_clip()
         if clip:
             self.effects_popover.set_relative_to(clip.ui)
             self.effects_popover.popup()
@@ -2105,6 +2152,7 @@ class TimelineContainer(Gtk.Grid, Zoomable, Loggable):
                 clips.extend(layer.get_clips())
 
         position = self._project.pipeline.get_position()
+        position = self.ges_timeline.get_frame_time(self.ges_timeline.get_frame_at(position))
         splitted = False
 
         with self._project.pipeline.commit_timeline_after():
@@ -2174,14 +2222,66 @@ class TimelineContainer(Gtk.Grid, Zoomable, Loggable):
         self.timeline.scroll_to_playhead(align=Gtk.Align.CENTER, when_not_in_view=True)
 
     def _seek_backward_one_frame_cb(self, unused_action, unused_parameter):
-        self._project.pipeline.step_frame(self._project.videorate, -1)
+        self._project.pipeline.step_frame(-1)
         self.timeline.scroll_to_playhead(align=Gtk.Align.CENTER, when_not_in_view=True)
 
     def _seek_forward_one_frame_cb(self, unused_action, unused_parameter):
 
-        self._project.pipeline.step_frame(self._project.videorate, 1)
+        self._project.pipeline.step_frame(1)
         self.timeline.scroll_to_playhead(align=Gtk.Align.CENTER, when_not_in_view=True)
 
+    def _shift_forward_cb(self, action: Gio.SimpleAction, parameter: Optional[GLib.Variant]) -> None:
+        self._shift_clips(1)
+
+    def _shift_backward_cb(self, action: Gio.SimpleAction, parameter: Optional[GLib.Variant]) -> None:
+        self._shift_clips(-1)
+
+    def _shift_clips(self, delta_frames):
+        """Shifts the selected clips position with the specified number of frames."""
+        if not self.ges_timeline:
+            return
+
+        previous_snapping_distance = self.ges_timeline.get_snapping_distance()
+        self.ges_timeline.set_snapping_distance(0)
+        try:
+            clips = list(self.timeline.selection.selected)
+            clips.sort(key=lambda candidate_clip: candidate_clip.start, reverse=delta_frames > 0)
+            # We must use delta * frame_time because getting negative frame time is not possible.
+            clip_delta = delta_frames * self.ges_timeline.get_frame_time(1)
+            self.app.action_log.begin("Shift clip delta frames")
+
+            for clip in clips:
+                if not clip.set_start(clip.start + clip_delta):
+                    self.app.action_log.rollback()
+                    return
+        finally:
+            self.ges_timeline.set_snapping_distance(previous_snapping_distance)
+        self.app.action_log.commit("Shift clip delta frames")
+
+    def _snap_clips_forward_cb(self, action, parameter):
+        self.snap_clips(forward=True)
+
+    def _snap_clips_backward_cb(self, action, parameter):
+        self.snap_clips(forward=False)
+
+    def snap_clips(self, forward: bool):
+        """Snap clips to next or previous clip."""
+        clips = list(self.timeline.selection.selected)
+        clips.sort(key=lambda clip: clip.start, reverse=forward)
+        with self.app.action_log.started("Snaps to closest clip",
+                                         finalizing_action=CommitTimelineFinalizingAction(self._project.pipeline),
+                                         toplevel=True):
+            for clip in clips:
+                layer = clip.props.layer
+                if not forward:
+                    position = self.first_clip_edge(layers=[layer], before=clip.start)
+                else:
+                    position = self.first_clip_edge(layers=[layer], after=clip.start + clip.duration)
+                    if position is not None:
+                        position -= clip.duration
+                if position is not None:
+                    clip.set_start(position)
+
     def do_focus_in_event(self, unused_event):
         self.log("Timeline has grabbed focus")
         self.update_actions()
