@@ -13,9 +13,8 @@
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, write to the
-# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
-# Boston, MA 02110-1301, USA.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
+"""Logic for undo/redo project actions."""
 from gi.repository import GES
 from gi.repository import GObject
 from gi.repository import Gst
@@ -66,7 +65,7 @@ class AssetAddedAction(Action):
         Action.__init__(self)
         self.asset = asset
 
-    def asScenarioAction(self):
+    def as_scenario_action(self):
         st = Gst.Structure.new_empty("add-asset")
         st.set_value("id", self.asset.get_id())
         type_string = GObject.type_name(self.asset.get_extractable_type())
@@ -88,7 +87,7 @@ class AssetRemovedAction(UndoableAction):
     def do(self):
         self.project.remove_asset(self.asset)
 
-    def asScenarioAction(self):
+    def as_scenario_action(self):
         st = Gst.Structure.new_empty("remove-asset")
         st.set_value("id", self.asset.get_id())
         type_string = GObject.type_name(self.asset.get_extractable_type())
@@ -132,18 +131,22 @@ class ProjectObserver(MetaContainerObserver):
 
     def __init__(self, project, action_log):
         MetaContainerObserver.__init__(self, project, action_log)
-        project.connect("asset-added", self._assetAddedCb)
-        project.connect("asset-removed", self._assetRemovedCb)
+        project.connect("asset-added", self._asset_added_cb)
+        project.connect("asset-removed", self._asset_removed_cb)
+        assets = project.list_assets(GES.Extractable)
+        for asset in assets:
+            MetaContainerObserver.__init__(self, asset, action_log)
         self.timeline_observer = TimelineObserver(project.ges_timeline,
                                                   action_log)
 
-    def _assetAddedCb(self, unused_project, asset):
+    def _asset_added_cb(self, unused_project, asset):
         if not isinstance(asset, GES.UriClipAsset):
             return
+        MetaContainerObserver.__init__(self, asset, self.action_log)
         action = AssetAddedAction(asset)
         self.action_log.push(action)
 
-    def _assetRemovedCb(self, project, asset):
+    def _asset_removed_cb(self, project, asset):
         if not isinstance(asset, GES.UriClipAsset):
             return
         action = AssetRemovedAction(project, asset)
