@@ -14,17 +14,13 @@
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, write to the
-# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
-# Boston, MA 02110-1301, USA.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
 from gi.repository import GES
 from gi.repository import GObject
 from gi.repository import Gst
 from gi.repository import Gtk
 
 from pitivi.utils.loggable import Loggable
-from pitivi.utils.ui import set_children_state_recurse
-from pitivi.utils.ui import unset_children_state_recurse
 
 
 # Selection modes
@@ -38,7 +34,6 @@ SELECT_ADD = 2
 
 class TimelineError(Exception):
     """Base Exception for errors happening in `Timeline`s or `Clip`s."""
-    pass
 
 
 class Selected(GObject.Object):
@@ -49,7 +44,7 @@ class Selected(GObject.Object):
     """
 
     __gsignals__ = {
-        "selected-changed": (GObject.SIGNAL_RUN_LAST, None, (bool,)),
+        "selected-changed": (GObject.SignalFlags.RUN_LAST, None, (bool,)),
     }
 
     def __init__(self):
@@ -83,7 +78,7 @@ class Selection(GObject.Object, Loggable):
     """
 
     __gsignals__ = {
-        "selection-changed": (GObject.SIGNAL_RUN_LAST, None, ()),
+        "selection-changed": (GObject.SignalFlags.RUN_LAST, None, ()),
     }
 
     def __init__(self):
@@ -93,7 +88,7 @@ class Selection(GObject.Object, Loggable):
         self.can_group = False
         self.can_ungroup = False
 
-    def setSelection(self, objs, mode):
+    def set_selection(self, objs, mode):
         """Updates the current selection.
 
         Args:
@@ -129,30 +124,30 @@ class Selection(GObject.Object, Loggable):
             obj.selected.selected = selected
             if obj.ui:
                 if selected:
+                    from pitivi.utils.ui import set_children_state_recurse
                     set_children_state_recurse(obj.ui, Gtk.StateFlags.SELECTED)
                 else:
+                    from pitivi.utils.ui import unset_children_state_recurse
                     unset_children_state_recurse(obj.ui, Gtk.StateFlags.SELECTED)
             for element in obj.get_children(False):
-                if isinstance(obj, GES.BaseEffect) or\
-                        isinstance(obj, GES.TextOverlay):
+                if isinstance(obj, (GES.BaseEffect, GES.TextOverlay)):
                     continue
                 element.selected.selected = selected
         self.set_can_group_ungroup()
         self.emit("selection-changed")
 
     def set_can_group_ungroup(self):
-        containers = set()
-        for obj in self.selected:
-            toplevel = obj.get_toplevel_parent()
-            if not toplevel.props.serialize:
-                for child in toplevel.get_children(False):
-                    containers.add(child)
-            else:
-                containers.add(toplevel)
-            if len(containers) > 1:
-                break
-        self.can_group = len(containers) > 1
-        self.can_ungroup = len(self.getSelectedTrackElements()) > 1
+        can_ungroup = False
+        toplevels = self.toplevels
+        self.can_group = len(toplevels) > 1
+        # If we allow grouping, we disallow ungrouping.
+        if not self.can_group:
+            for toplevel in toplevels:
+                if (isinstance(toplevel, GES.Group) or
+                        (isinstance(toplevel, GES.Container) and len(toplevel.get_children(False)) > 1)):
+                    can_ungroup = True
+                    break
+        self.can_ungroup = can_ungroup and not self.can_group
 
     def __get_selection_changes(self, old_selection):
         for obj in old_selection - self.selected:
@@ -165,12 +160,12 @@ class Selection(GObject.Object, Loggable):
             yield obj, True
 
     def select(self, objs):
-        self.setSelection(objs, SELECT)
+        self.set_selection(objs, SELECT)
 
     def unselect(self, objs):
-        self.setSelection(objs, UNSELECT)
+        self.set_selection(objs, UNSELECT)
 
-    def getSelectedTrackElements(self):
+    def get_selected_track_elements(self):
         """Returns the list of elements contained in this selection.
 
         Returns:
@@ -182,18 +177,7 @@ class Selection(GObject.Object, Loggable):
 
         return set(objects)
 
-    def getSelectedTrackElementsAtPosition(self, position, element_type=GObject.Object,
-                                           track_type=GES.TrackType.UNKNOWN):
-        selected = []
-        for clip in self.selected:
-            if clip.props.start <= position and position <= clip.props.start + clip.props.duration:
-                elements = clip.find_track_elements(None, track_type, element_type)
-                if elements:
-                    selected.extend(elements)
-
-        return selected
-
-    def getSingleClip(self, clip_type=GES.SourceClip):
+    def get_single_clip(self, clip_type=GES.SourceClip):
         """Returns the single-selected clip, if any.
 
         Args:
@@ -205,6 +189,49 @@ class Selection(GObject.Object, Loggable):
                 return clip
         return None
 
+    @property
+    def toplevels(self):
+        """Returns the toplevel elements of the selection."""
+        toplevels = set()
+        for obj in self.selected:
+            if not obj.timeline:
+                # The element has been removed from the timeline. Ignore it.
+                continue
+            toplevel = obj.get_toplevel_parent()
+            toplevels.update(self.__serializable_toplevels(toplevel))
+
+        return toplevels
+
+    def __serializable_toplevels(self, toplevel):
+        """Generates the specified element if serializable, or its children."""
+        if toplevel.props.serialize:
+            yield toplevel
+        else:
+            for element in toplevel.get_children(False):
+                for child in self.__serializable_toplevels(element):
+                    yield child
+
+    def group(self):
+        """Groups the serializable toplevel elements into a new GES.Group.
+
+        Returns:
+            GES.Group: The serializable elements which have no parent or
+            have only non-serializable ancestors.
+        """
+        toplevels = set()
+        for obj in self.selected:
+            if not obj.timeline:
+                # The element has been removed from the timeline. Ignore it.
+                continue
+            toplevel = obj.get_toplevel_parent()
+            toplevels.add(toplevel)
+
+        group = GES.Group.new()
+        group.props.serialize = False
+        for toplevel in toplevels:
+            group.add(toplevel)
+        return group
+
     def __len__(self):
         return len(self.selected)
 
@@ -243,9 +270,10 @@ class EditingContext(GObject.Object, Loggable):
         self.old_position = self.focus.get_start()
         if edge == GES.Edge.EDGE_END and mode == GES.EditMode.EDIT_TRIM:
             self.old_position += self.focus.get_duration()
-
         self.old_priority = self.focus.get_priority()
+
         self.new_position = None
+        self.new_priority = None
 
         self.timeline = timeline
         self.app = app
@@ -265,9 +293,9 @@ class EditingContext(GObject.Object, Loggable):
         if self.__log_actions:
             self.app.action_log.commit("move-clip")
         self.timeline.get_asset().pipeline.commit_timeline()
-        self.timeline.ui.app.gui.viewer.clipTrimPreviewFinished()
+        self.timeline.ui.app.gui.editor.viewer.clip_trim_preview_finished()
 
-    def setMode(self, mode):
+    def set_mode(self, mode):
         """Sets the current editing mode.
 
         Args:
@@ -292,26 +320,39 @@ class EditingContext(GObject.Object, Loggable):
         self.new_position = position
         self.new_priority = priority
 
-        res = self.focus.edit([], priority, self.mode, self.edge, int(position))
-        self.app.write_action("edit-container",
-            container_name=self.focus.get_name(),
-            position=float(position / Gst.SECOND),
-            edit_mode=self.mode.value_nick,
-            edge=self.edge.value_nick,
-            new_layer_priority=int(priority))
+        if self.with_video:
+            frame = self.timeline.get_frame_at(position)
+            time = self.timeline.get_frame_time(frame)
+            if position != time:
+                frame_dur = self.timeline.get_frame_time(1)
+                next_time = self.timeline.get_frame_time(frame + 1)
+                if abs(position - next_time) < frame_dur / 2:
+                    position = next_time
+                else:
+                    position = time
 
-        if res and self.mode == GES.EditMode.EDIT_TRIM and self.with_video:
-            if self.edge == GES.Edge.EDGE_START:
-                self.timeline.ui.app.gui.viewer.clipTrimPreview(self.focus, self.focus.props.in_point)
-            elif self.edge == GES.Edge.EDGE_END:
-                self.timeline.ui.app.gui.viewer.clipTrimPreview(self.focus,
-                                                                self.focus.props.duration + self.focus.props.in_point)
+        res = self.focus.edit([], priority, self.mode, self.edge, int(position))
+        if res:
+            self.app.write_action("edit-container",
+                                  container_name=self.focus.get_name(),
+                                  position=float(position / Gst.SECOND),
+                                  edit_mode=self.mode.value_nick,
+                                  edge=self.edge.value_nick,
+                                  new_layer_priority=int(priority))
+
+            if self.with_video:
+                if self.edge == GES.Edge.EDGE_START:
+                    self.timeline.ui.app.gui.editor.viewer.clip_trim_preview(
+                        self.focus, self.focus.props.in_point)
+                elif self.edge == GES.Edge.EDGE_END:
+                    self.timeline.ui.app.gui.editor.viewer.clip_trim_preview(
+                        self.focus, self.focus.props.duration + self.focus.props.in_point)
 
 
 # -------------------------- Interfaces ----------------------------------------#
 
 
-class Zoomable(object):
+class Zoomable:
     """Base class for conversions between timeline timestamps and UI pixels.
 
     Complex Timeline interfaces v2 (01 Jul 2008)
@@ -326,11 +367,11 @@ class Zoomable(object):
     ex : 0.1 = 1 pixel for 10 seconds
     ex : 1.0 = 1 pixel for a second
      Class Methods
-    . pixelToNs(pixels)
-    . nsToPixels(time)
-    . setZoomRatio
+    . pixel_to_ns(pixels)
+    . ns_to_pixels(time)
+    . set_zoom_ratio
     Instance Methods
-    . zoomChanged()
+    . zoom_changed()
     """
 
     sigid = None
@@ -346,9 +387,9 @@ class Zoomable(object):
 
     def __init__(self):
         # FIXME: ideally we should deprecate this
-        Zoomable.addInstance(self)
+        Zoomable.add_instance(self)
         if Zoomable.zoomratio is None:
-            Zoomable.zoomratio = self.computeZoomRatio(self._cur_zoom)
+            Zoomable.zoomratio = self.compute_zoom_ratio(self._cur_zoom)
 
     def __del__(self):
         if self in Zoomable._instances:
@@ -356,66 +397,61 @@ class Zoomable(object):
             self._instances.remove(self)
 
     @classmethod
-    def addInstance(cls, instance):
+    def add_instance(cls, instance):
         cls._instances.append(instance)
 
     @classmethod
-    def removeInstance(cls, instance):
+    def remove_instance(cls, instance):
         cls._instances.remove(instance)
 
     @classmethod
-    def setZoomRatio(cls, ratio):
+    def set_zoom_ratio(cls, ratio):
         ratio = min(max(cls.min_zoom, ratio), cls.max_zoom)
         if cls.zoomratio != ratio:
             cls.zoomratio = ratio
             for inst in cls._instances:
-                inst.zoomChanged()
+                inst.zoom_changed()
 
     @classmethod
-    def setZoomLevel(cls, level):
+    def set_zoom_level(cls, level):
         level = int(max(0, min(level, cls.zoom_steps)))
         if level != cls._cur_zoom:
             cls._cur_zoom = level
-            cls.setZoomRatio(cls.computeZoomRatio(level))
+            cls.set_zoom_ratio(cls.compute_zoom_ratio(level))
 
     @classmethod
-    def getCurrentZoomLevel(cls):
+    def get_current_zoom_level(cls):
         return cls._cur_zoom
 
     @classmethod
-    def zoomIn(cls):
-        cls.setZoomLevel(cls._cur_zoom + 1)
+    def zoom_in(cls):
+        cls.set_zoom_level(cls._cur_zoom + 1)
         cls.app.write_action("zoom-in", optional_action_type=True)
 
     @classmethod
-    def zoomOut(cls):
-        cls.setZoomLevel(cls._cur_zoom - 1)
+    def zoom_out(cls):
+        cls.set_zoom_level(cls._cur_zoom - 1)
         cls.app.write_action("zoom-out", optional_action_type=True)
 
     @classmethod
-    def computeZoomRatio(cls, x):
+    def compute_zoom_ratio(cls, x):
         return ((((float(x) / cls.zoom_steps) ** 3) * cls.zoom_range) +
                 cls.min_zoom)
 
     @classmethod
-    def computeZoomLevel(cls, ratio):
+    def compute_zoom_level(cls, ratio):
         return int((
             (max(0, ratio - cls.min_zoom) /
-                cls.zoom_range) ** (1.0 / 3.0)) * cls.zoom_steps)
+             cls.zoom_range) ** (1.0 / 3.0)) * cls.zoom_steps)
 
     @classmethod
-    def pixelToNs(cls, pixel):
+    def pixel_to_ns(cls, pixel):
         """Returns the duration equivalent of the specified pixel."""
         return int(pixel * Gst.SECOND / cls.zoomratio)
 
     @classmethod
-    def pixelToNsAt(cls, pixel, ratio):
-        """Returns the duration equivalent of the specified pixel."""
-        return int(pixel * Gst.SECOND / ratio)
-
-    @classmethod
-    def nsToPixel(cls, duration):
-        """Returns the pixel equivalent of the specified duration"""
+    def ns_to_pixel(cls, duration):
+        """Returns the pixel equivalent of the specified duration."""
         # Here, a long time ago (206f3a05), a pissed programmer said:
         # DIE YOU CUNTMUNCH CLOCK_TIME_NONE UBER STUPIDITY OF CRACK BINDINGS !!
         if duration == Gst.CLOCK_TIME_NONE:
@@ -423,13 +459,13 @@ class Zoomable(object):
         return int((float(duration) / Gst.SECOND) * cls.zoomratio)
 
     @classmethod
-    def nsToPixelAccurate(cls, duration):
+    def ns_to_pixel_accurate(cls, duration):
         """Returns the pixel equivalent of the specified duration."""
         # Here, a long time ago (206f3a05), a pissed programmer said:
         # DIE YOU CUNTMUNCH CLOCK_TIME_NONE UBER STUPIDITY OF CRACK BINDINGS !!
         if duration == Gst.CLOCK_TIME_NONE:
             return 0
-        return ((float(duration) / Gst.SECOND) * cls.zoomratio)
+        return (float(duration) / Gst.SECOND) * cls.zoomratio
 
-    def zoomChanged(self):
+    def zoom_changed(self):
         pass
