@@ -1,11 +1,8 @@
 #!/usr/bin/env python3
-"""
-Pitivi unit tests
-"""
-import glob
+# -*- coding: utf-8 -*-
+"""Pitivi unit tests."""
 import os
 import sys
-import unittest
 from tempfile import mkdtemp
 
 import gi.overrides
@@ -20,7 +17,6 @@ def get_pitivi_dir():
 
 def _prepend_env_paths(**args):
     """Prepends one or more paths to an environment variable."""
-
     for name, value in args.items():
         if not isinstance(value, list):
             value = [value]
@@ -59,15 +55,10 @@ def setup():
 
     # Make available the compiled C code.
     sys.path.append(configure.BUILDDIR)
-    subproject_paths = os.path.join(configure.BUILDDIR, "subprojects", "gst-transcoder")
 
-    _prepend_env_paths(LD_LIBRARY_PATH=subproject_paths,
-                       GST_PLUGIN_PATH=subproject_paths,
-                       GI_TYPELIB_PATH=subproject_paths,
-                       GST_PRESET_PATH=[os.path.join(pitivi_dir, "data", "videopresets"),
+    _prepend_env_paths(GST_PRESET_PATH=[os.path.join(pitivi_dir, "data", "videopresets"),
                                         os.path.join(pitivi_dir, "data", "audiopresets")],
-                       GST_ENCODING_TARGET_PATH=[os.path.join(pitivi_dir, "tests", "test-encoding-targets"),
-                                                 os.path.join(pitivi_dir, "data", "encoding-profiles")])
+                       GST_ENCODING_TARGET_PATH=[os.path.join(pitivi_dir, "data", "encoding-profiles")])
     os.environ.setdefault('PITIVI_TOP_LEVEL_DIR', pitivi_dir)
 
     # Make sure the modules are initialized correctly.
@@ -80,5 +71,6 @@ def setup():
 
     return res
 
+
 if not setup():
     raise ImportError("Could not setup testsuite")
