@@ -27,6 +27,7 @@ import sys
 import tempfile
 import traceback
 import unittest
+from typing import List
 from unittest import mock
 
 from gi.repository import Gdk
@@ -188,58 +189,89 @@ class CheckedOperationDuration:
         signal.alarm(0)
 
 
-def setup_project_with_clips(func, assets_names=None):
-    """Sets up a Pitivi instance with the specified assets on the timeline."""
-    if assets_names is None:
-        assets_names = {"1sec_simpsons_trailer.mp4"}
+def setup_project(assets_names: List[str]):
+    """Sets up a Pitivi instance with no assets on the timeline."""
+    assert isinstance(assets_names, list)
 
-    def wrapper(self):
-        with cloned_sample(*list(assets_names)):
-            self.app = create_pitivi()
-            self.project = self.app.project_manager.new_blank_project()
-            self.timeline = self.project.ges_timeline
-            self.layer = self.timeline.append_layer()
-            self.action_log = self.app.action_log
-            project = self.app.project_manager.current_project
-            self.timeline_container = TimelineContainer(self.app, editor_state=self.app.gui.editor.editor_state)
-            self.timeline_container.set_project(project)
+    def decorator(func):
+        nonlocal assets_names
 
-            timeline = self.timeline_container.timeline
-            timeline.app.project_manager.current_project = project
-            timeline.get_parent = mock.MagicMock(return_value=self.timeline_container)
-            uris = collections.deque([get_sample_uri(fname) for fname in assets_names])
-            mainloop = create_main_loop()
+        def wrapper(self):
+            with cloned_sample(*assets_names):
+                self.app = create_pitivi()
 
-            def loaded_cb(project, timeline):
-                project.add_uris([uris.popleft()])
+                self.timeline_container = TimelineContainer(self.app, editor_state=self.app.gui.editor.editor_state)
 
-            def progress_cb(project, progress, estimated_time):
-                if progress == 100:
-                    if uris:
-                        project.add_uris([uris.popleft()])
+                def loaded_cb(unused_pm, project):
+                    self.timeline_container.set_project(project)
+                    self.project = project
+                    self.timeline = project.ges_timeline
+                    layers = self.timeline.get_layers()
+                    if layers:
+                        self.layer = layers[0]
                     else:
-                        mainloop.quit()
+                        self.layer = self.timeline.append_layer()
 
-            project.connect_after("loaded", loaded_cb)
-            project.connect_after("asset-loading-progress", progress_cb)
-            mainloop.run()
+                self.app.project_manager.connect("new-project-loaded", loaded_cb)
+                self.app.project_manager.new_blank_project()
+                self.action_log = self.app.action_log
+                project = self.app.project_manager.current_project
 
-            project.disconnect_by_func(loaded_cb)
-            project.disconnect_by_func(progress_cb)
+                timeline = self.timeline_container.timeline
+                timeline.app.project_manager.current_project = project
+                timeline.get_parent = mock.MagicMock(return_value=self.timeline_container)
+                uris = collections.deque([get_sample_uri(fname) for fname in assets_names])
+                mainloop = create_main_loop()
 
-            assets = project.list_assets(GES.UriClip)
+                def project_loaded_cb(project, timeline):
+                    project.add_uris([uris.popleft()])
+
+                def progress_cb(project, progress, estimated_time):
+                    if progress == 100:
+                        if uris:
+                            project.add_uris([uris.popleft()])
+                        else:
+                            mainloop.quit()
+
+                project.connect_after("loaded", project_loaded_cb)
+                project.connect_after("asset-loading-progress", progress_cb)
+                mainloop.run()
+
+                project.disconnect_by_func(project_loaded_cb)
+                project.disconnect_by_func(progress_cb)
+
+                func(self)
+
+            del self.app
+            del self.project
+            del self.timeline
+            del self.layer
+            del self.action_log
+            del self.timeline_container
+
+        return wrapper
+
+    return decorator
+
+
+def setup_project_with_clips(assets_names: List[str]):
+    """Sets up a Pitivi instance with the specified assets on the timeline."""
+    # Ensure this method is not being used directly as a decorator.
+    assert isinstance(assets_names, list)
+
+    def decorator(func):
+        nonlocal assets_names
+
+        @setup_project(assets_names)
+        def wrapper(self):
+            assets = self.app.project_manager.current_project.list_assets(GES.UriClip)
             self.timeline_container.insert_assets(assets, self.timeline.props.duration)
 
             func(self)
 
-        del self.app
-        del self.project
-        del self.timeline
-        del self.layer
-        del self.action_log
-        del self.timeline_container
+        return wrapper
 
-    return wrapper
+    return decorator
 
 
 def setup_timeline(func):
@@ -281,6 +313,8 @@ def setup_clipproperties(func):
         self.transformation_box = self.clipproperties.transformation_expander
         self.transformation_box._new_project_loaded_cb(None, self.project)
 
+        self.speed_box = self.clipproperties.speed_expander
+
         func(self)
 
         del self.transformation_box
@@ -441,10 +475,44 @@ class TestCase(unittest.TestCase, Loggable):
 
     def assert_layers(self, layers):
         self.assertEqual(self.timeline.get_layers(), layers)
-        # Import TestLayers locally, otherwise its tests are discovered and
-        # run twice.
-        from tests.test_timeline_timeline import TestLayers
-        TestLayers.check_priorities_and_positions(self, self.timeline.ui, layers, list(range(len(layers))))
+        self.check_priorities_and_positions(self.timeline.ui, layers, list(range(len(layers))))
+
+    def check_priorities_and_positions(self, timeline, ges_layers,
+                                       expected_priorities):
+        layers_vbox = timeline.layout.layers_vbox
+
+        # Check the layers priorities.
+        priorities = [ges_layer.props.priority for ges_layer in ges_layers]
+        self.assertListEqual(priorities, expected_priorities)
+
+        # Check the positions of the Layer widgets.
+        positions = [layers_vbox.child_get_property(ges_layer.ui, "position")
+                     for ges_layer in ges_layers]
+        expected_positions = [priority * 2 + 1
+                              for priority in expected_priorities]
+        self.assertListEqual(positions, expected_positions, layers_vbox.get_children())
+
+        # Check the positions of the LayerControl widgets.
+        controls_vbox = timeline._layers_controls_vbox
+        positions = [controls_vbox.child_get_property(ges_layer.control_ui, "position")
+                     for ges_layer in ges_layers]
+        self.assertListEqual(positions, expected_positions)
+
+        # Check the number of the separators.
+        count = len(ges_layers) + 1
+        self.assertEqual(len(timeline._separators), count)
+        controls_separators, layers_separators = list(zip(*timeline._separators))
+
+        # Check the positions of the LayerControl separators.
+        expected_positions = [2 * index for index in range(count)]
+        positions = [layers_vbox.child_get_property(separator, "position")
+                     for separator in layers_separators]
+        self.assertListEqual(positions, expected_positions)
+
+        # Check the positions of the Layer separators.
+        positions = [controls_vbox.child_get_property(separator, "position")
+                     for separator in controls_separators]
+        self.assertListEqual(positions, expected_positions)
 
     def assert_effect_count(self, clip, count):
         effects = [effect for effect in clip.get_children(True)
