@@ -183,43 +183,6 @@ class TestLayers(common.TestCase):
             ges_layers.append(ges_layer)
         self.check_priorities_and_positions(timeline, ges_layers, expected_priorities)
 
-    def check_priorities_and_positions(self, timeline, ges_layers,
-                                       expected_priorities):
-        layers_vbox = timeline.layout.layers_vbox
-
-        # Check the layers priorities.
-        priorities = [ges_layer.props.priority for ges_layer in ges_layers]
-        self.assertListEqual(priorities, expected_priorities)
-
-        # Check the positions of the Layer widgets.
-        positions = [layers_vbox.child_get_property(ges_layer.ui, "position")
-                     for ges_layer in ges_layers]
-        expected_positions = [priority * 2 + 1
-                              for priority in expected_priorities]
-        self.assertListEqual(positions, expected_positions, layers_vbox.get_children())
-
-        # Check the positions of the LayerControl widgets.
-        controls_vbox = timeline._layers_controls_vbox
-        positions = [controls_vbox.child_get_property(ges_layer.control_ui, "position")
-                     for ges_layer in ges_layers]
-        self.assertListEqual(positions, expected_positions)
-
-        # Check the number of the separators.
-        count = len(ges_layers) + 1
-        self.assertEqual(len(timeline._separators), count)
-        controls_separators, layers_separators = list(zip(*timeline._separators))
-
-        # Check the positions of the LayerControl separators.
-        expected_positions = [2 * index for index in range(count)]
-        positions = [layers_vbox.child_get_property(separator, "position")
-                     for separator in layers_separators]
-        self.assertListEqual(positions, expected_positions)
-
-        # Check the positions of the Layer separators.
-        positions = [controls_vbox.child_get_property(separator, "position")
-                     for separator in controls_separators]
-        self.assertListEqual(positions, expected_positions)
-
     def test_remove_layer(self):
         self.check_remove_layer([0, 0, 0])
         self.check_remove_layer([0, 0, 1])
@@ -329,7 +292,7 @@ class TestGrouping(common.TestCase):
 
             self.assertEqual(clips[0].get_parent(), clip.get_parent())
             self.assertTrue(bool(clip.ui.get_state_flags() & Gtk.StateFlags.SELECTED))
-            self.assertTrue(clip.selected.selected)
+            self.assertTrue(clip.selected)
 
         group = clips[0].get_parent()
         self.assertEqual(len(group.get_children(False)), len(clips))
@@ -356,7 +319,7 @@ class TestGrouping(common.TestCase):
         # Make sure all the clips have been deselected
         for clip in clips:
             self.assertFalse(bool(clip.ui.get_state_flags() & Gtk.StateFlags.SELECTED))
-            self.assertFalse(clip.selected.selected)
+            self.assertFalse(clip.selected)
 
     def test_group_ungroup(self):
         num_clips = 2
@@ -376,11 +339,12 @@ class TestGrouping(common.TestCase):
             self.assertIsNone(clip.get_parent())
 
     def test_group_splitted_clip_and_select_group(self):
-        position = 5
 
         timeline_container = common.create_timeline_container()
         timeline = timeline_container.timeline
         clips = self.add_clips_simple(timeline, 1)
+        clips[0].props.duration = timeline.ges_timeline.get_frame_time(4)
+        position = timeline.ges_timeline.get_frame_time(2)
         self.toggle_clip_selection(clips[0], expect_selected=True)
 
         timeline.ges_timeline.get_asset().pipeline.get_position = mock.Mock(return_value=position)
@@ -393,8 +357,8 @@ class TestGrouping(common.TestCase):
 
         # Only the first clip is selected so select the
         # second one
-        self.assertTrue(clips[0].selected.selected)
-        self.assertFalse(clips[1].selected.selected)
+        self.assertTrue(clips[0].selected)
+        self.assertFalse(clips[1].selected)
 
         event = mock.Mock()
         event.keyval = Gdk.KEY_Control_L
@@ -405,7 +369,7 @@ class TestGrouping(common.TestCase):
         timeline_container.do_key_release_event(event)
 
         for clip in clips:
-            self.assertTrue(clip.selected.selected)
+            self.assertTrue(clip.selected)
 
         # Group the two parts
         timeline_container.group_action.emit("activate", None)
@@ -558,6 +522,34 @@ class TestCopyPaste(common.TestCase):
         timeline_container.paste_action.emit("activate", None)
         self.assertEqual(len(layer.get_clips()), 1)
 
+    def test_paste_selection(self):
+        timeline_container = self.copy_clips(1)
+        timeline = timeline_container.timeline
+        layer = timeline.ges_timeline.get_layers()[0]
+        project = timeline.ges_timeline.get_asset()
+
+        # Paste a single clip.
+        clips = layer.get_clips()
+        self.assertEqual(len(clips), 1)
+        position = 10
+        project.pipeline.get_position = mock.Mock(return_value=position)
+        timeline_container.paste_action.emit("activate", None)
+
+        clips = layer.get_clips()
+        self.assertListEqual([bool(clip.selected) for clip in clips], [False, True])
+
+        # The copy_clips() above simulates CTRL button press, so this will add 1st clip to the selection.
+        self.toggle_clip_selection(clips[0], True)
+        timeline_container.copy_action.emit("activate", None)
+
+        # Paste two clips.
+        position = 20
+        project.pipeline.get_position = mock.Mock(return_value=position)
+        timeline_container.paste_action.emit("activate", None)
+
+        clips = layer.get_clips()
+        self.assertListEqual([bool(clip.selected) for clip in clips], [False, False, True, True])
+
 
 class TestEditing(common.TestCase):
 
@@ -877,3 +869,157 @@ class TestDragFromOutside(common.TestCase):
         # Use same asset to mimic dragging multiple assets
         self.check_drag_assets_to_timeline(self.ges_timeline.ui, [asset, asset])
         self.assertEqual(layer.get_clips(), clips)
+
+
+class TestKeyboardShiftClips(common.TestCase):
+
+    def check_frame_shift_clips(self, *ges_clips):
+        """Checks that clips shifted forwards then backwards work properly."""
+        clip_original_starts = [clip.start for clip in ges_clips]
+        delta = self.timeline.get_frame_time(1)
+
+        event = mock.Mock()
+        event.keyval = Gdk.KEY_Control_L
+        self.timeline_container.do_key_press_event(event)
+        for clip in ges_clips:
+            self.toggle_clip_selection(clip, expect_selected=True)
+        self.timeline_container.do_key_release_event(event)
+
+        self.timeline_container.shift_forward_action.emit("activate", None)
+        self.assertListEqual([clip.start - delta for clip in ges_clips], clip_original_starts)
+
+        self.timeline_container.shift_backward_action.emit("activate", None)
+        self.assertListEqual([clip.start for clip in ges_clips], clip_original_starts)
+
+    @common.setup_project(["tears_of_steel.webm"])
+    def test_clip_shift(self):
+        """Checks that shift methods change position of a single clip by one frame."""
+        ges_clip1 = self.add_clip(self.layer, 5 * Gst.SECOND, duration=2 * Gst.SECOND)
+        self.check_frame_shift_clips(ges_clip1)
+
+    @common.setup_project(["tears_of_steel.webm"])
+    def test_shift_disjoint_clips(self):
+        """Checks that disjoint clips are able to be shifted."""
+        ges_clip1 = self.add_clip(self.layer, 5 * Gst.SECOND, duration=1 * Gst.SECOND)
+        ges_clip2 = self.add_clip(self.layer, 9 * Gst.SECOND, duration=1 * Gst.SECOND)
+        self.check_frame_shift_clips(ges_clip1, ges_clip2)
+
+    @common.setup_project(["tears_of_steel.webm"])
+    def test_shift_adjacent_clips(self):
+        """Checks that adjacent clips are able to be shifted as well."""
+        ges_clip1 = self.add_clip(self.layer, 5 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip2 = self.add_clip(self.layer, 7 * Gst.SECOND, duration=2 * Gst.SECOND)
+        self.check_frame_shift_clips(ges_clip1, ges_clip2)
+
+    @common.setup_project(["tears_of_steel.webm"])
+    def test_triple_overlap_causes_rollback(self):
+        """Checks that rollback works properly in the event of triple overlap."""
+        ges_clip1 = self.add_clip(self.layer, 5 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip2 = self.add_clip(self.layer, 10 * Gst.SECOND, duration=2 * Gst.SECOND)
+        self.add_clip(self.layer, start=11 * Gst.SECOND, duration=2 * Gst.SECOND)
+        self.add_clip(self.layer, start=12 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip3 = self.add_clip(self.layer, 13 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip4 = self.add_clip(self.layer, 15 * Gst.SECOND, duration=2 * Gst.SECOND)
+
+        event = mock.Mock()
+        event.keyval = Gdk.KEY_Control_L
+        self.timeline_container.do_key_press_event(event)
+        self.toggle_clip_selection(ges_clip1, expect_selected=True)
+        self.toggle_clip_selection(ges_clip2, expect_selected=True)
+        self.toggle_clip_selection(ges_clip3, expect_selected=True)
+        self.toggle_clip_selection(ges_clip4, expect_selected=True)
+
+        self.timeline_container.do_key_release_event(event)
+
+        self.timeline_container.shift_forward_action.emit("activate", None)
+
+        self.assertEqual(5 * Gst.SECOND, ges_clip1.start)
+        self.assertEqual(10 * Gst.SECOND, ges_clip2.start)
+        self.assertEqual(13 * Gst.SECOND, ges_clip3.start)
+        self.assertEqual(15 * Gst.SECOND, ges_clip4.start)
+
+        self.timeline_container.shift_backward_action.emit("activate", None)
+
+        self.assertEqual(5 * Gst.SECOND, ges_clip1.start)
+        self.assertEqual(10 * Gst.SECOND, ges_clip2.start)
+        self.assertEqual(13 * Gst.SECOND, ges_clip3.start)
+        self.assertEqual(15 * Gst.SECOND, ges_clip4.start)
+
+
+class TestSnapClips(common.TestCase):
+
+    @common.setup_timeline
+    def test_snap_clip_single_layer_single_clip(self):
+        """Test whether a single clip is able to snap right, and left to an adjacent clip."""
+        ges_clip1 = self.add_clip(self.layer, 5 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip2 = self.add_clip(self.layer, 9 * Gst.SECOND, duration=2 * Gst.SECOND)
+
+        self.toggle_clip_selection(ges_clip1, expect_selected=True)
+
+        self.timeline_container.snap_clips_forward_action.emit("activate", None)
+        self.assertEqual(ges_clip1.start, ges_clip2.start - ges_clip1.duration)
+
+        self.timeline_container.snap_clips_backward_action.emit("activate", None)
+        self.assertEqual(ges_clip1.start, 0)
+
+    @common.setup_timeline
+    def test_snap_single_layer_multiple_clips_adjacent(self):
+        """Tests whether a single clip can snap to multiple adjacent clips."""
+        ges_clip1 = self.add_clip(self.layer, 5 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip2 = self.add_clip(self.layer, 7 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip3 = self.add_clip(self.layer, 11 * Gst.SECOND, duration=2 * Gst.SECOND)
+
+        event = mock.Mock()
+        event.keyval = Gdk.KEY_Control_L
+        self.timeline_container.do_key_press_event(event)
+        self.toggle_clip_selection(ges_clip1, expect_selected=True)
+        self.toggle_clip_selection(ges_clip2, expect_selected=True)
+
+        self.timeline_container.snap_clips_forward_action.emit("activate", None)
+        self.assertEqual(ges_clip2.start, ges_clip3.start - ges_clip2.duration)
+        self.assertEqual(ges_clip1.start, ges_clip2.start - ges_clip1.duration)
+
+        self.timeline_container.snap_clips_backward_action.emit("activate", None)
+        self.assertEqual(ges_clip1.start, 0)
+        self.assertEqual(ges_clip2.start, ges_clip1.start + ges_clip1.duration)
+
+    @common.setup_timeline
+    def test_snap_multiple_layers_not_affected_by_other_layer(self):
+        """Tests whether a clip snap is affected by a clip in another layer."""
+        layer2 = self.timeline.append_layer()
+        self.add_clip(self.layer, 5 * Gst.SECOND, duration=2 * Gst.SECOND)
+        self.add_clip(self.layer, 7 * Gst.SECOND, duration=2 * Gst.SECOND)
+        self.add_clip(self.layer, 11 * Gst.SECOND, duration=2 * Gst.SECOND)
+        clip = self.add_clip(layer2, 5 * Gst.SECOND, duration=2 * Gst.SECOND)
+        end_clip = self.add_clip(layer2, 30 * Gst.SECOND, duration=2 * Gst.SECOND)
+
+        self.toggle_clip_selection(clip, expect_selected=True)
+
+        self.timeline_container.snap_clips_forward_action.emit("activate", None)
+        self.assertEqual(clip.start, end_clip.start - clip.duration)
+
+        self.timeline_container.snap_clips_backward_action.emit("activate", None)
+        self.assertEqual(clip.start, 0)
+
+    @common.setup_timeline
+    def test_single_clip_snap_right_does_nothing(self):
+        """Tests whether the last clip snapped forward remains in place."""
+        ges_clip = self.add_clip(self.layer, 5 * Gst.SECOND, duration=2 * Gst.SECOND)
+        clip_start = ges_clip.start
+
+        self.toggle_clip_selection(ges_clip, expect_selected=True)
+
+        self.timeline_container.snap_clips_forward_action.emit("activate", None)
+        self.assertEqual(ges_clip.start, clip_start)
+
+    @common.setup_timeline
+    def test_clip_snaps_to_timeline_duration(self):
+        """Tests whether a clip snaps to the timeline duration."""
+        layer2 = self.timeline.append_layer()
+        self.add_clip(self.layer, 30 * Gst.SECOND, duration=2 * Gst.SECOND)
+        ges_clip = self.add_clip(layer2, 5 * Gst.SECOND, duration=2 * Gst.SECOND)
+
+        self.toggle_clip_selection(ges_clip, expect_selected=True)
+
+        self.timeline_container.snap_clips_forward_action.emit("activate", None)
+        self.assertEqual(ges_clip.start, self.timeline.get_duration() - ges_clip.duration)
