@@ -13,9 +13,7 @@
 # Lesser General Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, write to the
-# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
-# Boston, MA 02110-1301, USA.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
 from unittest import mock
 
 from gi.repository import GES
@@ -31,7 +29,7 @@ from tests import common
 
 class TestSelected(common.TestCase):
 
-    def testBoolEvaluation(self):
+    def test_bool_evaluation(self):
         selected = Selected()
         self.assertFalse(selected)
 
@@ -44,60 +42,93 @@ class TestSelected(common.TestCase):
 
 class TestSelection(common.TestCase):
 
-    def testBoolEvaluation(self):
+    def test_bool_evaluation(self):
         clip1 = mock.MagicMock()
         selection = Selection()
         self.assertFalse(selection)
-        selection.setSelection([clip1], SELECT)
+        selection.set_selection([clip1], SELECT)
         self.assertTrue(selection)
-        selection.setSelection([clip1], SELECT_ADD)
+        selection.set_selection([clip1], SELECT_ADD)
         self.assertTrue(selection)
-        selection.setSelection([clip1], UNSELECT)
+        selection.set_selection([clip1], UNSELECT)
         self.assertFalse(selection)
 
-    def testGetSingleClip(self):
+    def test_get_single_clip(self):
         selection = Selection()
         clip1 = common.create_test_clip(GES.UriClip)
         clip2 = common.create_test_clip(GES.TitleClip)
 
         # Selection empty.
-        self.assertIsNone(selection.getSingleClip())
-        self.assertIsNone(selection.getSingleClip(GES.UriClip))
-        self.assertIsNone(selection.getSingleClip(GES.TitleClip))
+        self.assertIsNone(selection.get_single_clip())
+        self.assertIsNone(selection.get_single_clip(GES.UriClip))
+        self.assertIsNone(selection.get_single_clip(GES.TitleClip))
 
-        selection.setSelection([clip1], SELECT)
-        self.assertEqual(selection.getSingleClip(), clip1)
-        self.assertEqual(selection.getSingleClip(GES.UriClip), clip1)
-        self.assertIsNone(selection.getSingleClip(GES.TitleClip))
+        selection.set_selection([clip1], SELECT)
+        self.assertEqual(selection.get_single_clip(), clip1)
+        self.assertEqual(selection.get_single_clip(GES.UriClip), clip1)
+        self.assertIsNone(selection.get_single_clip(GES.TitleClip))
 
-        selection.setSelection([clip2], SELECT)
-        self.assertEqual(selection.getSingleClip(), clip2)
-        self.assertIsNone(selection.getSingleClip(GES.UriClip))
-        self.assertEqual(selection.getSingleClip(GES.TitleClip), clip2)
+        selection.set_selection([clip2], SELECT)
+        self.assertEqual(selection.get_single_clip(), clip2)
+        self.assertIsNone(selection.get_single_clip(GES.UriClip))
+        self.assertEqual(selection.get_single_clip(GES.TitleClip), clip2)
 
-        selection.setSelection([clip1, clip2], SELECT)
-        self.assertIsNone(selection.getSingleClip())
-        self.assertIsNone(selection.getSingleClip(GES.UriClip))
-        self.assertIsNone(selection.getSingleClip(GES.TitleClip))
+        selection.set_selection([clip1, clip2], SELECT)
+        self.assertIsNone(selection.get_single_clip())
+        self.assertIsNone(selection.get_single_clip(GES.UriClip))
+        self.assertIsNone(selection.get_single_clip(GES.TitleClip))
 
     def test_can_group_ungroup(self):
-        clip1 = common.create_test_clip(GES.UriClip)
-        clip2 = common.create_test_clip(GES.UriClip)
+        timeline_container = common.create_timeline_container()
+        timeline = timeline_container.timeline
+        clip1, clip2 = self.add_clips_simple(timeline, 2)
+
         selection = Selection()
-        self.assertFalse(selection)
-
-        selection.setSelection([clip1], SELECT)
-        self.assertFalse(selection.can_ungroup)
         self.assertFalse(selection.can_group)
+        self.assertFalse(selection.can_ungroup)
 
-        selection.setSelection([clip2], SELECT_ADD)
+        selection.set_selection([clip1], SELECT)
+        self.assertFalse(selection.can_group)
+        self.assertTrue(selection.can_ungroup)
+
+        selection.set_selection([clip2], SELECT_ADD)
         self.assertTrue(selection.can_group)
         self.assertFalse(selection.can_ungroup)
 
-        selection.setSelection([], SELECT)
+        selection.set_selection([], SELECT)
         self.assertFalse(selection.can_group)
         self.assertFalse(selection.can_ungroup)
 
+    def test_toplevels(self):
+        timeline_container = common.create_timeline_container()
+        timeline = timeline_container.timeline
+        clip1, clip2, clip3, clip4 = self.add_clips_simple(timeline, 4)
+
+        selection = Selection()
+
+        selection.set_selection([clip1, clip2, clip3, clip4], SELECT)
+        self.assertSetEqual(selection.toplevels, {clip1, clip2, clip3, clip4})
+
+        group1 = GES.Container.group([clip1, clip2])
+        group1.props.serialize = True
+        self.assertSetEqual(selection.toplevels, {group1, clip3, clip4})
+
+        group2 = GES.Container.group([group1, clip3])
+        group2.props.serialize = True
+        self.assertSetEqual(selection.toplevels, {group2, clip4})
+
+        group1.props.serialize = True
+        group1.props.serialize = False
+        self.assertSetEqual(selection.toplevels, {group2, clip4})
+
+        group1.props.serialize = False
+        group2.props.serialize = False
+        self.assertSetEqual(selection.toplevels, {clip1, clip2, clip3, clip4})
+
+        group1.props.serialize = True
+        group2.props.serialize = False
+        self.assertSetEqual(selection.toplevels, {group1, clip3, clip4})
+
 
 class TestEditingContext(common.TestCase):
     """Tests for the EditingContext class."""
@@ -111,9 +142,10 @@ class TestEditingContext(common.TestCase):
         # Add the clips to a layer so they have TrackElements.
         project = common.create_project()
         layer = project.ges_timeline.append_layer()
-        layer.add_clip(audio_clip)
-        layer.add_clip(video_clip)
-        layer.add_clip(audio_video_clip)
+        self.assertTrue(layer.add_clip(audio_clip))
+        self.assertTrue(layer.add_clip(video_clip))
+        audio_video_clip.props.start = video_clip.props.duration
+        self.assertTrue(layer.add_clip(audio_video_clip))
 
         self.__check_with_video(audio_clip, False)
         self.__check_with_video(video_clip, True)
